close all
instrreset
clear all

% Code for taking data from the HP 8564E Spectrum Analyzer using the HP
% 83650B Swept Signal Generator and the HP 33120A waveform generator
% User specified the resonances to be measured in line 23 from the
% ResList.mat file along with the filename extension and the number of
% synchronized and unsynchronized sweeps.  Be careful to use unique file 
% name extensions, or previous data can be over-written.

% The code features a number of successive zoom-ins to correctly locate a
% resonance and center it on the screen all using the synchronized mode to
% save time.  The coding may not be the most efficient and has some
% redundancy in it, but it works.

T1=input('Starting Temperature in deg. C : ');
P1=input('Starting gauge Pressure in bars (-1 for vacuum) : ');
flnm='vac3';
%flnm='nh3_halfbar_1percent';
%flnm='nh3_3bars_premix';
%for R=85:89        % use for K/Ka-band Fabry-Perot measurements
for R=60:70         % use for small cavity resonator measurements

%for R=50:56        % high frequency resonances in the large resonator

instrreset          % clear previously open GPIB instrument objects

numswp=10;          % number of synchronized sweeps
numswp2=5;          % number of unsychronized sweep-on-scans

load ResList        % load the center frequency, approximate zoomed-in span, RBW, bigspan, and power level for each resonance
if R > 0 && R < 92
    
sig = gpib('ni',0,19);      % swept signal generator
SA = gpib('ni',0,18);       % spectrum analyzer
WFG = gpib('ni',0,10);      % waveform generator (used for sychronization)
SA.InputBufferSize=4300;    % necessary for retrieving length of 601 points of data
fopen(sig)
fopen(SA)
fopen(WFG)
set(SA,'Timeout',50)

SweepTime=0.975;            % spec. ana. sweep time necessary for synchronization
if R>71                     % Fabry-Perot (at room temp)
    F=Flist(R)*1e9;
else
    F=Flist(R)*1e9*14698/14737;  % small cylindrical cavity resonator scaled for whatever temperature
end

RBW=RBWlist(R);
pwr=Pwrlist(R);
bigspan=BigSpanlist(R)*1e6;
SigSpan=SigSpanlist(R)*1e6;
if F < 6.4e9                    % mixer integers from spectrum analyzer documentation
    MixN=1;
elseif F < 13.2e9
    MixN=2;
elseif F < 31e9
    MixN=4;
elseif F < 40.05e9
    MixN=8;
end

VBW=RBW/10;             % unsynchronized video bandwidth
RBW2=RBW*10;            % synchronized resolution bandwidth
VBW2=RBW2/10;           % synchronized video bandwidth

    if R==60
        beep
        pause(0.25)
        beep
        fprintf(sig, 'POW:STAT OFF')
        input('connect small resonator with K-band/Pasternack isolator')
    end  

    if R==50
        beep
        pause(0.25)
        beep
        fprintf(sig, 'POW:STAT OFF')
        input('connect large resonator with Pasternack isolator')
    end      

    if R==85
        beep
        pause(0.25)
        beep
        fprintf(sig, 'POW:STAT OFF')
        input('connect ka-band resonator')
    end    

fprintf(WFG, 'APPL:SQU 0.55, 0.5, 0.25') %turn trigger off
fprintf(SA, 'BLANK TRB;CLRW TRA;')      % blank trace B and clear/write to trace A

fprintf(sig, 'POW:STAT ON')             % turn on RF output from sig. gen.
fprintf(sig, 'SWE:FREQ:GEN ANAL')       % set frequency sweep type
fprintf(sig, 'SWE:FREQ:TIME 1')         % set sig. gen. sweep time
fprintf(sig, 'POW:LEVEL %f', pwr)       % set sig. gen. output power level (dBm)

fprintf(sig, 'INIT:CONT 1')             % set sig. gen. sweep type to continuous (as opposed to single)
fprintf(SA, 'DET POS;')                 % set spec. ana. detection mode to positive peak
fprintf(SA, 'AT 0;')                    % set spec. ana. attenuation level to 0 dB
fprintf(SA, 'RL -12;')                  % set spec. ana. reference level to -12 dB
fprintf(SA, 'LG 5DB;')                  % set spec. ana. scale to 5 dB/division

fprintf(sig, 'FREQ:CENT %f', F)         % set sig. gen. center frequency (Hz)
fprintf(sig, 'FREQ:SPAN %f', bigspan*1.015) % set sig. gen. span (Hz)
pause(0.5)                              % wait half second
fprintf(sig, 'FREQ:SPAN %f', bigspan*1.015) % set sig. gen. span again because it doesn't always take hold the first time
fprintf(SA, 'CF %f;', F)            % set spec. ana. center frequency (Hz)
fprintf(SA, 'SP %f;', bigspan)      % set spec. ana. wide span (Hz)
fprintf(SA, 'RB 2e6')               % set spec. ana. Resolution Bandwidth (Hz) 
fprintf(SA, 'VB %f', VBW2)          % set spec. ana. Video Bandwidth (Hz) 
fprintf(SA, 'ST 0.975S;')           % set spec. ana. sweep time (s)

fprintf(SA, 'CONTS;')           % set spec. ana. sweep type to continuous

fprintf(SA, 'TM EXT')           % set spec. ana. triggering mode to externally triggered
fprintf(sig, 'TRIG:SOUR EXT;')  % set sig. gen. triggering mode to externally triggered
fprintf(WFG, 'APPL:SQU 0.55, 2.5, 1.25')  % turn on waveform generator to trigger other devices
pause(7)                        % allow time for triggering to lock and trace to stabilize
fprintf(SA,'SNGLS;TS;')         % take a single sweep from spectrum analyzer
fprintf(SA, 'CONTS;')           % return spec. ana. to continuous sweep mode
pause(0.5)
fprintf(SA, 'MKPK HI;')         % place a marker on the peak of the trace
if R==70||R==69||R==66||R==67   % list of resonances that require manual location
   beep
   pause(0.25)
   beep
   fclose(SA)                   % closing the spec. ana. GBIP connection to allow control by the user
   input('manually tune to resonance')         % manually move marker to desired peak
   fopen(SA)                    % reopen spec. ana. GBIP link 
end
fprintf(SA, 'MKCF;')            % Make the center frequency that of the marker
fprintf(SA,'CF?;')              % query center frequency
F = fscanf(SA, '%f');           % read in center frequency
fprintf(sig, 'FREQ:CENT %f', F)     % set sig. gen. center frequency
fprintf(sig, 'FREQ:SPAN %f', 100e6*1.015) % set sig. gen. span scaled for synchronization
pause(0.5)
fprintf(sig, 'FREQ:SPAN %f', 100e6*1.015) % set sig. gen. span again 
fprintf(SA, 'SP %f;', 100e6)    % set spec. ana. span to 100 MHz
fprintf(SA, 'RB 2e6')           % set spec. ana. RBW to max of 2 MHz
fprintf(SA, 'ST 0.975S;')       % set spec. ana. sweep time to 0.975 seconds
fprintf(SA, 'CONTS;')           % put spec. ana. in continuous sweep mode
pause(7)                        % wait for synchronization to lock

fprintf(SA,'SNGLS;TS;')
fprintf(SA,'TDF P;TRA?;')
A = fscanf(SA);
Bwide=strread(A,'%f','delimiter',',');
fprintf(SA, 'CONTS;')
freqwide=linspace(F-50e6, F+50e6, 601);     % establish frequency vector

fprintf(SA, 'SP %f;', SigSpan)
fprintf(sig, 'FREQ:CENT %f', F)
fprintf(sig, 'FREQ:SPAN %f', SigSpan*1.015)
pause(0.5)
fprintf(sig, 'FREQ:SPAN %f', SigSpan*1.015)
fprintf(SA, 'RB %f;', RBW2)
pause(7)

fprintf(SA,'SNGLS;TS;')
fprintf(SA, 'MKPK HI;MKA?;')    % query amplitude of marker
AmpSync = fscanf(SA, '%f');     % read in peak value of trace
AmpSync = AmpSync+1-rem(AmpSync,1); % truncate peak number to nearest dB + 1
fprintf(SA, 'MKCF;')
pause(0.5)
fprintf(SA,'CF?;')
F = fscanf(SA, '%f');


fprintf(sig, 'FREQ:CENT %f', F)
fprintf(sig, 'FREQ:SPAN %f', SigSpan*1.015)
pause(0.5)
fprintf(sig, 'FREQ:SPAN %f', SigSpan*1.015)

fprintf(SA, 'RL %f;', AmpSync)
fprintf(SA, 'LG 1DB;')
fprintf(SA, 'CONTS;')
pause(7)

fprintf(SA,'SNGLS;TS;')
fprintf(SA,'TDF P;TRA?;')       % query floating point data from trace A
A = fscanf(SA);                 % read in data
B=strread(A,'%f','delimiter',',');  % parse data
freq=linspace(F-SigSpan/2, F+SigSpan/2, 601);   % set up accompanying frequency points

[bw3db PeakF]=quickprocess_sync(B,freq);        % quickly locate frequency of the peak and full-width at half-max
bw3db=bw3db-rem(bw3db,1e4);
PeakF=PeakF-rem(PeakF,1e4);

fprintf(SA, 'SP %f;', bw3db*2)      % make spec. ana. span twice the 3-dB bandwidth
fprintf(SA, 'CF %f;', PeakF)
fprintf(SA, 'ST %f;', SweepTime)
fprintf(sig, 'FREQ:CENT %f', PeakF)
fprintf(sig, 'FREQ:SPAN %f', bw3db*2*1.015)
pause(0.5)
fprintf(sig, 'FREQ:SPAN %f', bw3db*2*1.015)

fprintf(SA, 'CONTS;')
fprintf(SA, 'RL %f;', AmpSync+3)
pause(7)

fprintf(SA,'SNGLS;TS;')
fprintf(SA,'TDF P;TRA?;')
A = fscanf(SA);
B=strread(A,'%f','delimiter',',');
fprintf(SA, 'MKPK HI;MKA?;')
AmpSync2 = fscanf(SA, '%f');
AmpSync2 = AmpSync2+1-rem(AmpSync2,1);

fprintf(SA, 'RL %f;', AmpSync2)
fprintf(SA, 'CONTS;')
freq=linspace(PeakF-bw3db, PeakF+bw3db, 601);
[bw3db PeakF]=quickprocess_sync(B,freq);
bw3db=bw3db-rem(bw3db,1e4);
PeakF=PeakF-rem(PeakF,1e4);

SweTime=SweepTime;
if bw3db>(1e6*MixN)
    if bw3db<(1.3e6*MixN)
        bw3db=(1e6*MixN);
    else
        SweTime=0.975;
    end
else
    SweTime=SweepTime;
end

fprintf(SA, 'ST %f;', SweTime)
fprintf(SA, 'CF %f;', PeakF)
fprintf(SA, 'SP %f;', bw3db*2)
fprintf(SA,'CF?;')
PeakF = fscanf(SA, '%f');
fprintf(sig, 'FREQ:CENT %f', PeakF)
fprintf(sig, 'FREQ:SPAN %f', bw3db*2*1.015)
pause(0.5)
fprintf(sig, 'FREQ:SPAN %f', bw3db*2*1.015)

fprintf(SA, 'RB %f;', RBW)
fprintf(SA, 'VB %f', VBW)
fprintf(SA, 'TM FREE;')
fprintf(sig, 'TRIG:SOUR IMM;')
fprintf(sig, 'SWE:FREQ:TIME 0.075;') 
fprintf(SA, 'ST 10;')
fprintf(sig, 'INIT:CONT 1')
pause(7)
fprintf(SA,'SNGLS;TS;')
fprintf(SA,'TDF P;TRA?;')
A = fscanf(SA);
B=strread(A,'%f','delimiter',',');
fprintf(SA, 'MKPK HI;MKA?;')
Amp = fscanf(SA, '%f');
Amp = Amp+1-rem(Amp,1);
fprintf(SA, 'RL %f;', Amp)

fprintf(SA, 'ST 40;')
fprintf(SA,'CF?;')
CenterFreq = fscanf(SA, '%f');
fprintf(SA,'AT?;')
Attenuation = fscanf(SA, '%f');
fprintf(SA,'DET?;')
Detector = fscanf(SA, '%c');
fprintf(SA,'SP?;')
Span = fscanf(SA, '%f');
fprintf(SA,'RB?;')
RBW = fscanf(SA, '%f');
fprintf(SA,'RL?;')
RL = fscanf(SA, '%f');
fprintf(SA,'VB?;')
VBW = fscanf(SA, '%f');
freq=linspace(CenterFreq-Span/2, CenterFreq+Span/2, 601);
figure(R)
plot(freqwide,Bwide)

if R==67||R==68||R==66||R==61   % set a number of default stabilization 
                                % wasted sweeps for resonances that require
                                % or frequencies that take time to
                                % stabilize
stablesweepnum=8;
elseif R==85||R==86
stablesweepnum=15;
elseif R==69||R==60
stablesweepnum=9;
end

fprintf(sig, 'INIT:CONT 1')
if R==66||R==67||R==68||R==85||R==86||R==69||R==60

long_errcnt=0;
q=1;
while q<=stablesweepnum         % take sweeps that will be discarded, just to allow resonance amplitudes to stabilize
    long_sweep_stabilizing = q
    fprintf(sig, 'INIT:CONT 1')
    fprintf(SA,'SNGLS;')
    pause(45)
    fprintf(SA,'TDF P;TRA?;')
    A = fscanf(SA);
    B(:,q)=strread(A,'%f','delimiter',',');
    fprintf(SA, 'CONTS;')
    gooddata=0;
    for k=1:601
        if B(k,q)>RL-10
            gooddata=gooddata+1;
        end
    end
    if gooddata >= 130  % detects wether or not the sig. gen. stopped working mid-sweep, which it's prone to doing sometimes
        q=q+1;
    else                % restarts the sig. gen. and gets it to lock back onto continuous sweep mode
        fprintf(sig, 'SWE:MODE MAN')
        pause(1)
        fprintf(sig, 'FREQ:CENT %f', CenterFreq)
        pause(1)
        fprintf(sig, 'FREQ:CENT %f', CenterFreq)
        fprintf(sig, 'FREQ:SPAN %f', Span*1.015)
        pause(1)
        fprintf(sig, 'FREQ:SPAN %f', Span*1.015)
        pause(1)
        fprintf(sig, 'INIT:CONT 0')
        pause(1)
        fprintf(sig, 'INIT:CONT 1')
        pause(1)
        fprintf(sig, 'SWE:MODE AUTO')
        pause(1)
        fprintf(sig, 'TRIG:SOUR IMM;')
    end
end
clear B
end

% Take unsynchronized data sweeps

long_errcnt=0;
q=1;
while q<=numswp2
    long_sweep = q
    fprintf(sig, 'INIT:CONT 1')
    fprintf(SA,'SNGLS;')
    pause(45)
    fprintf(SA,'TDF P;TRA?;')
    A = fscanf(SA);
    B(:,q)=strread(A,'%f','delimiter',',');
    fprintf(SA, 'CONTS;')
    gooddata=0;
    for k=1:601
        if B(k,q)>RL-10
            gooddata=gooddata+1;
        end
    end
    if gooddata >= 130
        q=q+1;
    else
        fprintf(sig, 'SWE:MODE MAN')
        pause(1)
        fprintf(sig, 'FREQ:CENT %f', CenterFreq)
        pause(1)
        fprintf(sig, 'FREQ:CENT %f', CenterFreq)
        fprintf(sig, 'FREQ:SPAN %f', Span*1.015)
        pause(1)
        fprintf(sig, 'FREQ:SPAN %f', Span*1.015)
        pause(1)
        fprintf(sig, 'INIT:CONT 0')
        pause(1)
        fprintf(sig, 'INIT:CONT 1')
        pause(1)
        fprintf(sig, 'SWE:MODE AUTO')
        pause(1)
        fprintf(sig, 'TRIG:SOUR IMM;')
    end
end

figure(100*R)
plot(freq,B)


% Take synchronized data sweeps

fprintf(SA, 'RL %f;', AmpSync2)
fprintf(WFG, 'APPL:SQU 0.55, 2.5, 1.25')
    fprintf(SA, 'TM EXT')
    fprintf(sig, 'TRIG:SOUR EXT;') 
    fprintf(sig, 'SWE:FREQ:TIME 1;') 
    fprintf(SA, 'ST %f;', SweTime)
    fprintf(SA, 'RB %f;', RBW2)
    
    fprintf(sig, 'SWE:FREQ:GEN ANAL')
    fprintf(SA, 'CONTS;')
    fprintf(sig, 'INIT:CONT 1')
Thresh=AmpSync2-10;         % set threshhold for determining proper synchronization

while Thresh<AmpSync2-9 
for i=1:3
    sweep = i
    fprintf(SA,'SNGLS;TS;')
    fprintf(SA,'TDF P;TRA?;')
    A = fscanf(SA);
    fprintf(SA, 'CONTS;')
    C(:,i)=strread(A,'%f','delimiter',',');
    peak(i)=max(C(:,i));
end
Thresh=max(peak)
if Thresh >= AmpSync2-0.5
    fprintf(SA, 'RL %f;', AmpSync2+1)
end
end

threshtol=0.2

i=1;
errcnt=0;
while i<numswp+1
    sweep = i
    fprintf(SA,'SNGLS;TS;')
    fprintf(SA,'TDF P;TRA?;')
    A = fscanf(SA);
    fprintf(SA, 'CONTS;')
    C(:,i)=strread(A,'%f','delimiter',',');
    peak2=max(C(:,i));
    if peak2>(Thresh-threshtol) 
        i=i+1;
    else
        if errcnt==5
            Thresh=AmpSync2-10;
            pause(10)
            errcnt=0;
            while Thresh<AmpSync2-9
                for i=1:5
                    sweep = i
                    fprintf(SA,'SNGLS;TS;')
                    fprintf(SA,'TDF P;TRA?;')
                    A = fscanf(SA);
                    fprintf(SA, 'CONTS;')
                    C(:,i)=strread(A,'%f','delimiter',',');
                    peak(i)=max(C(:,i));
                end
            Thresh=max(peak)
            end
            i=1;
        end
        
        errcnt=errcnt+1
        beep
    end
end
fclose(SA)
fclose(sig)
fclose(WFG)

figure(100*R)
hold on
plot(freq,C)
else
    disp('invalid input for resonance #');
end

time=fix(clock);

P2=P1;
T2=T1;
if mod(R,1)==0
pause(0.5)
beep
pause(0.25)
beep
T2=input('Finishing Temperature in deg. C : ');
P2=input('Finishing gauge Pressure in bars (-1 for vacuum) : ');
end

%P2=-1;     % uncomment for vacuum measurements


timestamp=num2str(time(6)+1e2*time(5)+1e4*time(4)+1e6*time(3)+1e8*time(2)+1e10*time(1));
Rs=num2str(R);
filename=strcat(Rs,'_', flnm);
clear *Str *0
save (filename)

T1=T2;
P1=P2;

end
pause(0.25)
beep                % alert the user that the measurements are complete
pause(0.25)
beep
pause(0.25)
beep
pause(0.25)
beep