close all
instrreset
clear all

% code for taking transmissivity measurements using the Agilent 5071C-ENA network analyzer

% the user specifies the appropriate file name extensions in the beginning
% and end of the code along with the lists of measurements to iterate
% through and the number of resonances.  The number of the transmissivity
% measurement (1, 2, or 3) is specified in line 12.

Transnum='1';

for R=1:19
for pressures=1:14

    Rstr=num2str(R);
    if pressures==1
        filenm=strcat(Rstr, '_nh3_30mbar_pure');
    elseif pressures==2
        filenm=strcat(Rstr, '_nh3_4bars_quarterpercent');
    elseif pressures==3
        filenm=strcat(Rstr, '_nh3_8bars_quarterpercent');
    elseif pressures==4
        filenm=strcat(Rstr, '_nh3_12bars_quarterpercent');
    elseif pressures==5
        filenm=strcat(Rstr, '_nh3_8bars_16thpercent');
    elseif pressures==6
        filenm=strcat(Rstr, '_nh3_12bars_16thpercent');   
    elseif pressures==7
        filenm=strcat(Rstr, '_nh3_120mbar_pure');
    elseif pressures==8
        filenm=strcat(Rstr, '_nh3_12bars_1percent');
    elseif pressures==9
        filenm=strcat(Rstr, '_nh3_8bars_1percent');
    elseif pressures==10
        filenm=strcat(Rstr, '_nh3_6bars_1percent');
    elseif pressures==11
        filenm=strcat(Rstr, '_nh3_4bars_1percent');
    elseif pressures==12
        filenm=strcat(Rstr, '_nh3_2bars_1percent');
    elseif pressures==13
        filenm=strcat(Rstr, '_nh3_1bar_1percent');   
    elseif pressures==14
        filenm=strcat(Rstr, '_nh3_halfbar_1percent');           
    end  
    % next line is used for excluding measurements that don't exist
 if (pressures==1||pressures==7||pressures==8||pressures==9||pressures==10) && R==19
 else
    load (filenm)
    clear S21 S12
    numpoints=1601;
    peak=0;
    SweepTime2=20;
    numswp=10;

    instrreset
    NA = gpib('ni',0,17);
    NA.InputBufferSize=250000;
    fclose(NA)
    fopen(NA)
    fprintf(NA, ':OUTP ON')
    fprintf(NA, ':SENS1:CORR:CLE')              % clear calibration correction coefficient
    fprintf(NA, ':SENS1:SWE:DEL 0.01')          % sweep delay set to 10ms
    fprintf(NA, ':SENS1:SWE:GEN STEP')          % sweep set to stepped mode
    fprintf(NA, ':SOUR1:POW 0')                 % set source power to 0 dBm
    fprintf(NA, ':CALC1:FORM MLOG')             % log magnitude format 
    
fprintf(NA, ':CALC1:PAR1:DEF S21')              % measure S(2,1)
fprintf(NA, ':SENS1:FREQ:CENT %f;', faxis(801))          % set center frequency
fprintf(NA, ':SENS1:FREQ:SPAN %f;', (faxis(1601)-faxis(1)))    % set frequency span
fprintf(NA, ':SENS1:BWID %f;', IFBW)            % Set IF bandwidth
fprintf(NA, ':SENS1:SWE:POIN %i;', numpoints)   % set # of points
fprintf(NA, ':SENS1:SWE:TIME:AUTO ON')          % turn on auto sweep time
fprintf(NA, ':SENS1:SWE:TIME?')                 % query sweep time
Swetime = fscanf(NA, '%f');
pause(0.5+Swetime)
fprintf(NA, ':DISP:WIND1:TRAC1:Y:AUTO')     % set display power axis to auto scale
fprintf(NA, ':SENS1:FREQ:DATA?')                % query frequency data
faxisStr = fscanf(NA);
faxis=strread(faxisStr,'%f','delimiter',',');
for j=1:numswp                                  % measure S(2,1)
    fprintf(NA, ':CALC1:DATA:FDAT?')
    S21Str = fscanf(NA);
    S210=strread(S21Str,'%f','delimiter',',');
    for i=1:2*numpoints
        if mod((i+1),2)==0
            S21((i+1)/2,j)=S210(i);
        end
    end
    pause(Swetime)
end
fprintf(NA, ':CALC1:PAR1:DEF S12')              % measure S(1,2)
pause(Swetime+0.5)
fprintf(NA, ':DISP:WIND1:TRAC1:Y:AUTO')     % set display power axis to auto scale
for j=1:numswp                                  
    fprintf(NA, ':CALC1:DATA:FDAT?')
    S12Str = fscanf(NA);
    S120=strread(S12Str,'%f','delimiter',',');
    for i=1:2*numpoints
        if mod((i+1),2)==0
            S12((i+1)/2,j)=S120(i);
        end
    end
    pause(Swetime)
end
fprintf(NA, ':CALC1:PAR1:DEF S22')              % measure S(2,2)
pause(Swetime+0.5)
fprintf(NA, ':DISP:WIND1:TRAC1:Y:AUTO')     % set display power axis to auto scale
fprintf(NA, ':CALC1:DATA:FDAT?')
S22Str = fscanf(NA);
S220=strread(S22Str,'%f','delimiter',',');
for i=1:2*numpoints
    if mod((i+1),2)==0
        S22((i+1)/2)=S220(i);
    end
end
pause(Swetime)
fprintf(NA, ':CALC1:PAR1:DEF S11')              % measure S(1,1)
pause(Swetime+0.5)
fprintf(NA, ':DISP:WIND1:TRAC1:Y:AUTO')     % set display power axis to auto scale
fprintf(NA, ':CALC1:DATA:FDAT?')
S11Str = fscanf(NA);
S110=strread(S11Str,'%f','delimiter',',');
for i=1:2*numpoints
    if mod((i+1),2)==0
        S11((i+1)/2)=S110(i);
    end
end
fclose(NA)


%figure(R*100+pressures)  % can be uncommented to display a new window for
                          % each mesurement, but will quickly overwhelm the CPU
subplot(2,2,1)
plot(faxis,S11)
title('S(1,1)')
subplot(2,2,2)
plot(faxis,S12)
title('S(1,2)')
subplot(2,2,3)
plot(faxis,S21)
title('S(2,1)')
subplot(2,2,4)
plot(faxis,S22)
title('S(2,2)')


time=fix(clock);
timestamp=num2str(time(6)+1e2*time(5)+1e4*time(4)+1e6*time(3)+1e8*time(2)+1e10*time(1));
Rs=num2str(R);

    if pressures==1
        filename=strcat(Rs,'_trans',Transnum, '_30mbar_pure');
    elseif pressures==2
        filename=strcat(Rs,'_trans',Transnum, '_4bars_quarterpercent');
    elseif pressures==3
        filename=strcat(Rs,'_trans',Transnum, '_8bars_quarterpercent');
    elseif pressures==4
        filename=strcat(Rs,'_trans',Transnum, '_12bars_quarterpercent');
    elseif pressures==5
        filename=strcat(Rs,'_trans',Transnum, '_8bars_16thpercent');
    elseif pressures==6
        filename=strcat(Rs,'_trans',Transnum, '_12bars_16thpercent');   
    elseif pressures==7
        filename=strcat(Rs,'_trans',Transnum, '_120mbar_pure');
    elseif pressures==8
        filename=strcat(Rs,'_trans',Transnum, '_12bars_1percent');
    elseif pressures==9
        filename=strcat(Rs,'_trans',Transnum, '_8bars_1percent');
    elseif pressures==10
        filename=strcat(Rs,'_trans',Transnum, '_6bars_1percent');
    elseif pressures==11
        filename=strcat(Rs,'_trans',Transnum, '_4bars_1percent');
    elseif pressures==12
        filename=strcat(Rs,'_trans',Transnum, '_2bars_1percent');
    elseif pressures==13
        filename=strcat(Rs,'_trans',Transnum, '_1bar_1percent');   
    elseif pressures==14
        filename=strcat(Rs,'_trans',Transnum, '_halfbar_1percent');           
    end      
clear *Str *0
save (filename)
end
end
hold off
end
